@@ -1,4 +1,4 @@
-/*  $Id: french-conjugator.cpp,v 1.16 2009/02/08 03:42:14 sarrazip Exp $
+/*  $Id: french-conjugator.cpp,v 1.17 2009/10/04 03:03:34 sarrazip Exp $
     french-conjugator.cpp - Conjugation of French verbs
 
     verbiste - French conjugation system
@@ -54,6 +54,7 @@ static struct option knownOptions[] =
     { "template",       required_argument,      NULL, 'l' },
     { "pronouns",       no_argument,            NULL, 'p' },
     { "utf8",           no_argument,            NULL, 'u' },
+    { "all-infinitives",no_argument,            NULL, 'i' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
@@ -243,6 +244,7 @@ displayHelp()
 "--template=T       Use template T to conjugate the verbs\n"
 "--pronouns         Include pronouns in the displayed conjugation\n"
 "--utf8             Assume that the terminal uses UTF-8 instead of Latin-1\n"
+"--all-infinitives  Print the names of all known verbs, one per line (unsorted)\n"
 "\n"
 "See the " << commandName << "(1) manual page for details.\n"
 "\n"
@@ -264,6 +266,7 @@ main(int argc, char *argv[])
     Tense reqTense = INVALID_TENSE;
     string reqTemplate;
     bool includePronouns = false;
+    bool listAllInfinitives = false;
 
     #ifdef HAVE_GETOPT_LONG
 
@@ -317,6 +320,10 @@ main(int argc, char *argv[])
 	    	utf8Mode = true;
 		break;
 
+	    case 'i':
+		listAllInfinitives = true;
+		break;
+
 	    default:
 		displayHelp();
 		return EXIT_FAILURE;
@@ -335,6 +342,10 @@ main(int argc, char *argv[])
     #endif  /* ndef HAVE_GETOPT_LONG */
 
     ConjugatorCommand cmd(utf8Mode);
+    
+    if (listAllInfinitives)
+	return cmd.listAllInfinitives(cout);
+
     cmd.reqMode = reqMode;
     cmd.reqTense = reqTense;
     cmd.reqTemplate = reqTemplate;
