@@ -1,4 +1,4 @@
-/*  $Id: french-deconjugator.cpp,v 1.14 2009/02/08 03:42:14 sarrazip Exp $
+/*  $Id: french-deconjugator.cpp,v 1.15 2009/10/04 03:03:34 sarrazip Exp $
     french-deconjugator.cpp - Analyzer of conjugated French verbs
 
     verbiste - French conjugation system
@@ -50,6 +50,7 @@ static struct option knownOptions[] =
     { "help",          no_argument,       NULL, 'h' },
     { "version",       no_argument,       NULL, 'v' },
     { "utf8",          no_argument,       NULL, 'u' },
+    { "all-infinitives",no_argument,      NULL, 'i' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
@@ -147,6 +148,7 @@ displayHelp()
 "--help             Display this help page and exit\n"
 "--version          Display this program's version number and exit\n"
 "--utf8             Assume that the terminal uses UTF-8 instead of Latin-1\n"
+"--all-infinitives  Print the names of all known verbs, one per line (unsorted)\n"
 "\n"
 "See the " << commandName << "(1) manual page for details.\n"
 "\n"
@@ -159,6 +161,7 @@ main(int argc, char *argv[])
 {
     setlocale(LC_CTYPE, "");
     bool utf8Mode = (strcmp(nl_langinfo(CODESET), "UTF-8") == 0);
+    bool listAllInfinitives = false;
 
     #ifdef HAVE_GETOPT_LONG
 
@@ -186,6 +189,10 @@ main(int argc, char *argv[])
 	    	utf8Mode = true;
 		break;
 
+	    case 'i':
+		listAllInfinitives = true;
+		break;
+
 	    default:
 		displayHelp();
 		return EXIT_FAILURE;
@@ -200,5 +207,8 @@ main(int argc, char *argv[])
 
     DeconjugatorCommand cmd(utf8Mode);
 
+    if (listAllInfinitives)
+	return cmd.listAllInfinitives(cout);
+
     return cmd.run(argc - optind, argv + optind);
 }
