@@ -1,4 +1,4 @@
-/*  $Id: panel-applet.cpp,v 1.13 2008/02/03 07:10:58 sarrazip Exp $
+/*  $Id: panel-applet.cpp,v 1.14 2009/03/10 02:59:36 sarrazip Exp $
     panel-applet.cpp - GNOME Panel applet main functions
 
     verbiste - French conjugation system
@@ -43,7 +43,7 @@ static GtkWidget *appletEntry = NULL;
 
 static
 void
-onAbout(BonoboUIComponent *uic, gpointer user_data, const char *verbname)
+onAbout(BonoboUIComponent * /*uic*/, gpointer /*user_data*/, const char * /*verbname*/)
 {
     showAbout();
 }
@@ -83,7 +83,7 @@ onKeyPressInEntry(GtkWidget *entry, GdkEventKey *event, gpointer)
 
 static
 gboolean
-entry_button_press_event(GtkWidget *widget,
+entry_button_press_event(GtkWidget * /*widget*/,
 			GdkEventButton *event,
 			gpointer *applet)
 {
