@@ -1,4 +1,4 @@
-/*  $Id: main-window.cpp,v 1.13 2009/03/10 03:03:52 sarrazip Exp $
+/*  $Id: main-window.cpp,v 1.18 2009/10/10 20:02:57 sarrazip Exp $
     main-window.cpp - Input and conjugation window
 
     verbiste - French conjugation system
@@ -57,7 +57,7 @@ FrenchVerbDictionary *fvd = NULL;
 static GtkWidget *resultWin = NULL;
 static GtkWidget *verbEntry = NULL;
 static GtkWidget *conjButton = NULL;
-static GtkWidget *saveButton = NULL;
+static GtkWidget *saveAsButton = NULL;
 static GtkWidget *showPronounsCB = NULL;
 static GtkWidget *resultNotebook = NULL;
 
@@ -292,7 +292,7 @@ onConjugateButton(GtkWidget *, gpointer)
 // Escape for HTML, including converting newline into <br/>.
 static
 string
-esc(const string &s)
+esc_html(const string &s)
 {
     string out;
     string::size_type len = s.length();
@@ -310,66 +310,34 @@ esc(const string &s)
     return out;
 }
 
-
+// Escape for LaTeX, for now just converting newline into "\\"
+// (is more needed here?)
 static
-void
-onSaveButton(GtkWidget *, gpointer)
+string
+esc_latex(const string &s)
 {
-    // Show a Save dialog that points to the last save dir, if available.
-    GtkWidget *chooser = gtk_file_chooser_dialog_new(
-				_("Save Conjugation to HTML File"),
-				GTK_WINDOW(resultWin),
-				GTK_FILE_CHOOSER_ACTION_SAVE,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-				NULL);
-    gtk_file_chooser_set_do_overwrite_confirmation(
-				GTK_FILE_CHOOSER(chooser),
-				true);
-
-    Catena saveDir = get_config_string("Files/SaveDirectory");
-    if (saveDir.get() != NULL)
-	gtk_file_chooser_set_current_folder(
-				    GTK_FILE_CHOOSER(chooser),
-				    saveDir.get());
-
-    gtk_file_chooser_set_current_name(
-				GTK_FILE_CHOOSER(chooser),
-				_("Untitled conjugation.html"));
-
-    bool cancel = (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_CANCEL);
-    string filename;
-    if (!cancel)
+    string out;
+    string::size_type len = s.length();
+    for (string::size_type i = 0; i < len; i++)
     {
-	Catena f = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-	filename = f.get();
-
-	// Remember the chosen directory:
-	Catena dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
-	set_config_string("Files/SaveDirectory", dir.get());
-
-	// We do not call sync_config() because it will be done by quit()
-	// when the program quits.
+	switch (s[i])
+	{
+	    case '\n': out += "\\\\\n"; break;
+	    default:  out += s[i];
+	}
     }
-    gtk_widget_destroy(chooser);
-    if (cancel)
-	return;
+    return out;
+}
 
-    // Try to create the file:
-
-    ofstream html(filename.c_str(), ios::out);
-    if (!html)
-    {
-	showErrorDialog(_("Could not create file:") + ("\n" + filename));
-	return;
-    }
 
+void writeHTMLConjugation(ostream &html)
+{
     html << "<?xml version='1.0'?>\n";
     html << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
 	    << " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
     html << "<html>\n";
     html << "<head>\n";
-    html << "<title>" << esc(gtk_entry_get_text(GTK_ENTRY(verbEntry)))
+    html << "<title>" << esc_html(gtk_entry_get_text(GTK_ENTRY(verbEntry)))
 	    << "</title>\n";
     html << "<meta http-equiv='Content-Type'"
 	    << " content='text/html; charset=UTF-8' />\n";
@@ -394,7 +362,7 @@ onSaveButton(GtkWidget *, gpointer)
 	g_return_if_fail(GTK_IS_LABEL(label));
 
 	const gchar *infinitive = gtk_label_get_text(GTK_LABEL(label));
-	html << "<h1>" << esc(infinitive) << "</h1>\n";
+	html << "<h1>" << esc_html(infinitive) << "</h1>\n";
 	html << "<table border='1' cellspacing='0' cellpadding='4'>\n";
 	string tr = "<tr valign='top'>\n";
 	html << tr;
@@ -429,8 +397,8 @@ onSaveButton(GtkWidget *, gpointer)
 	    g_return_if_fail(persons != NULL);
 
 	    html << "<td>\n";
-	    html << "<strong>" << esc(name) << "</strong>\n<br/>\n";
-	    html << esc(persons) << "\n";
+	    html << "<strong>" << esc_html(name) << "</strong>\n<br/>\n";
+	    html << esc_html(persons) << "\n";
 	    html << "</td>\n";
 
 	    if (childCounter == 0)
@@ -463,9 +431,265 @@ onSaveButton(GtkWidget *, gpointer)
 
     html << "</body>\n";
     html << "</html>\n";
+}
 
-    html.close();
-    if (!html)
+
+static
+void
+writeLaTeXConjugation(ostream &tex)
+{
+    tex << "% verbiste output - do not edit\n";
+    tex << "% " << esc_latex(gtk_entry_get_text(GTK_ENTRY(verbEntry))) << "\n";
+    tex << "\\documentclass{verbiste}\n";
+    tex << "\\title{" << esc_latex(gtk_entry_get_text(GTK_ENTRY(verbEntry))) << "}\n";
+    tex << "\\begin{document}\n";
+    tex << "\\thispagestyle{empty}\n";
+
+    assert(resultNotebook != NULL);
+    GtkWidget *w;
+    for (int i = 0; (w = gtk_notebook_get_nth_page(
+			GTK_NOTEBOOK(resultNotebook), i)) != NULL;
+		    i++)
+    {
+	ResultPage *rp = (ResultPage *) g_object_get_data(
+						G_OBJECT(w), "ResultPage");
+	g_return_if_fail(rp != NULL);
+	g_return_if_fail(GTK_IS_TABLE(rp->table));
+
+	GtkWidget *label = gtk_notebook_get_tab_label(
+					    GTK_NOTEBOOK(resultNotebook),
+					    rp->notebookPage);
+	g_return_if_fail(GTK_IS_LABEL(label));
+
+	string btab = "\\begin{tense}\n";
+	string etab = "\\end{tense}\n";
+	string eol  = "\\\\\n";
+	string eor  = "\\\\ \\hline\n";
+	string sep  = "&\n";
+
+	const gchar *infinitive = gtk_label_get_text(GTK_LABEL(label));
+	tex << "\\section*{" << esc_latex(infinitive) << "}\n";
+	tex << "\\noindent\n";
+	tex << "\\begin{tensetable}\n";
+
+	GList *children = gtk_container_get_children(GTK_CONTAINER(rp->table));
+	children = g_list_reverse(children);
+	int childCounter = 0;
+	for (GList *p = children; p != NULL; p = p->next, childCounter++)
+	{
+	    GtkWidget *vbox = (GtkWidget *) p->data;
+	    g_return_if_fail(vbox != NULL);
+	    g_return_if_fail(GTK_IS_VBOX(vbox));
+
+	    GList *vboxChildren = gtk_container_get_children(
+						GTK_CONTAINER(vbox));
+	    g_return_if_fail(vboxChildren != NULL);
+	    g_return_if_fail(vboxChildren->next != NULL);
+	    g_return_if_fail(vboxChildren->next->next == NULL);
+
+	    GtkWidget *nameLabel = (GtkWidget *) vboxChildren->data;
+	    GtkWidget *personsLabel = (GtkWidget *) vboxChildren->next->data;
+
+	    g_return_if_fail(nameLabel != NULL);
+	    g_return_if_fail(GTK_IS_LABEL(nameLabel));
+	    g_return_if_fail(personsLabel != NULL);
+	    g_return_if_fail(GTK_IS_LABEL(personsLabel));
+
+	    const gchar *name = gtk_label_get_text(GTK_LABEL(nameLabel));
+	    const gchar *persons = gtk_label_get_text(GTK_LABEL(personsLabel));
+
+	    g_return_if_fail(name != NULL);
+	    g_return_if_fail(persons != NULL);
+
+	    tex << btab;
+	    tex << "\\tensename{" << esc_latex(name) << "}" << eol;
+	    tex << esc_latex(persons) << "\n";
+	    tex << etab;
+
+	    switch (childCounter)
+	      {
+	      case 0:
+		tex << sep << sep << sep << eor;
+		break;
+	      case 4:
+		tex << eor;
+		break;
+	      case 7:
+	      case 10:
+		tex << sep << eor;
+		break;
+	      default:
+		tex << sep;
+	      }
+
+	    g_list_free(vboxChildren);
+	}
+	g_list_free(children);
+
+	tex << "\\end{tensetable}\n";
+    }
+
+    tex << "\\end{document}\n";
+}
+
+
+enum SaveFileFormat { HTML, LATEX, NUMFORMATS };
+
+
+const char *getSaveFileFormatExtension(SaveFileFormat fmt)
+{
+    switch (fmt)
+    {
+	case HTML: return ".html";
+	case LATEX: return ".tex";
+	case NUMFORMATS: return NULL;
+    }
+    return NULL;
+}
+
+
+static
+void
+onSaveFormatComboBoxChanged(GtkComboBox *comboBox, gpointer chooserWidget)
+{
+    // Get the file extension that corresponds to the currently selected format.
+    gint index = gtk_combo_box_get_active(GTK_COMBO_BOX(comboBox));
+    const char *newExt = getSaveFileFormatExtension(SaveFileFormat(index));
+    if (newExt == NULL)  // if invalid choice, bail out
+	return;
+
+    // Get the currently selected filename (full path).
+    GtkWidget *chooser = reinterpret_cast<GtkWidget *>(chooserWidget);
+    Catena f = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+
+    // Find the last slash and period in that filename.
+    // If the last period comes before the last slash, then the basename
+    // has no extension.
+    string newFilename = f.get();
+    string::size_type posLastPeriod = newFilename.rfind('.');
+    if (posLastPeriod == string::npos)
+	return;  // no extension found
+    string::size_type posLastSlash  = newFilename.rfind('/');
+    if (posLastSlash != string::npos && posLastPeriod < posLastSlash)
+	return;  // extension is not in basename
+
+    // Replace the current extension with the selected one.
+    newFilename.replace(posLastPeriod, newFilename.length() - posLastPeriod, newExt);
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), newFilename.c_str());
+}
+
+
+static
+void
+onSaveAsButton(GtkWidget *, gpointer)
+{
+    // Show a Save dialog that points to the last save dir, if available.
+    GtkWidget *chooser = gtk_file_chooser_dialog_new(
+				_("Save Conjugation to File"),
+				GTK_WINDOW(resultWin),
+				GTK_FILE_CHOOSER_ACTION_SAVE,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				NULL);
+    gtk_file_chooser_set_do_overwrite_confirmation(
+				GTK_FILE_CHOOSER(chooser),
+				true);
+
+
+    // Add a pull-down menu that offers to choose the file format.
+
+    GtkWidget *comboBox = gtk_combo_box_new_text();
+    // These appends must be in same order as enum SaveFileFormat:
+    gtk_combo_box_append_text(GTK_COMBO_BOX(comboBox), _("HTML"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(comboBox), _("LaTeX"));
+
+    // Get the default format from the saved configuration.
+    Catena saveFormat = get_config_string("Files/SaveFormat");
+    gint formatIndex = saveFormat.atoi();
+    SaveFileFormat defaultFormat = SaveFileFormat(formatIndex < 0 || formatIndex >= NUMFORMATS ? 0 : formatIndex);
+
+    gtk_combo_box_set_active(GTK_COMBO_BOX(comboBox), gint(defaultFormat));
+
+    // Connect the "changed" signal of the combo box.
+    g_signal_connect(G_OBJECT(comboBox), "changed",
+		G_CALLBACK(onSaveFormatComboBoxChanged), chooser);
+
+    GtkWidget *prompt = gtk_label_new_with_mnemonic(_("_Format of the saved file:"));
+    gtk_label_set_mnemonic_widget(GTK_LABEL(prompt), comboBox);
+
+    // Add the combo box and its prompt to the file chooser dialog.
+    GtkWidget *formatBox = gtk_hbox_new(FALSE, SP);
+    gtk_box_pack_start(GTK_BOX(formatBox), prompt, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(formatBox), comboBox, TRUE, TRUE, 0);
+
+    gtk_widget_show_all(GTK_WIDGET(formatBox));
+
+    gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(chooser), formatBox);
+
+
+    // Set the default directory from the saved configuration.
+    Catena saveDir = get_config_string("Files/SaveDirectory");
+    if (saveDir.get() != NULL)
+	gtk_file_chooser_set_current_folder(
+				    GTK_FILE_CHOOSER(chooser),
+				    saveDir.get());
+
+
+    // Base default filename on entered verb:
+    string defaultFilename = gtk_entry_get_text(GTK_ENTRY(verbEntry))
+				+ string(getSaveFileFormatExtension(defaultFormat));
+    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser), defaultFilename.c_str());
+
+
+    bool cancel = (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_CANCEL);
+    string filename;
+    SaveFileFormat chosenFormat = NUMFORMATS;  // invalid value for now
+    if (!cancel)
+    {
+	Catena f = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+	filename = f.get();
+
+	gint index = gtk_combo_box_get_active(GTK_COMBO_BOX(comboBox));
+	chosenFormat = SaveFileFormat(index < 0 || index >= NUMFORMATS ? 0 : index);
+
+	// Remember the chosen directory and format:
+	Catena dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
+	set_config_string("Files/SaveDirectory", dir.get());
+	stringstream ss;
+	ss << int(chosenFormat);
+	set_config_string("Files/SaveFormat", ss.str().c_str());
+
+	// We do not call sync_config() because it will be done by quit()
+	// when the program quits.
+    }
+    gtk_widget_destroy(chooser);
+    if (cancel)
+	return;
+
+    // Try to create the file:
+
+    ofstream file(filename.c_str(), ios::out);
+    if (!file)
+    {
+	showErrorDialog(_("Could not create file:") + ("\n" + filename));
+	return;
+    }
+
+    switch (chosenFormat)
+    {
+	case HTML:
+	    writeHTMLConjugation(file);
+	    break;
+	case LATEX:
+	    writeLaTeXConjugation(file);
+	    break;
+	case NUMFORMATS:
+	    assert(!"invalid chosen format");
+	    break;
+    }
+
+    file.close();
+    if (!file)
     {
 	showErrorDialog(_("Could not close file:") + ("\n" + filename));
 	return;
@@ -475,11 +699,11 @@ onSaveButton(GtkWidget *, gpointer)
 
 static
 GtkWidget *
-newLabel(const string &markup, gboolean selectable)
+newLabel(const string &markup, gboolean selectable, gfloat yalign = 0.0)
 {
     GtkWidget *label = gtk_label_new("");
     gtk_label_set_markup(GTK_LABEL(label), markup.c_str());
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, yalign);
     gtk_label_set_selectable(GTK_LABEL(label), selectable);
     return label;
 }
@@ -537,6 +761,43 @@ getResultWindowSize(gint &width, gint &height)
 }
 
 
+static
+void
+onAccentedCharClicked(GtkWidget *, gpointer a)
+{
+    // Insert UTF-8 string designated by 'a' in Verb text field,
+    // at current cursor position.
+    //
+    const char *utf8AccentedChar = reinterpret_cast<char *>(a);
+    gint currentPos = gtk_editable_get_position(GTK_EDITABLE(verbEntry));
+    gtk_editable_insert_text(GTK_EDITABLE(verbEntry), utf8AccentedChar, -1, &currentPos);
+
+    // Put focus on text field, so user can immediately
+    // go back to using keyboard.
+    // This must be done before setting cursor position,
+    // because setting focus selects entire text field contents.
+    // Setting position will unselect.
+    //
+    gtk_widget_grab_focus(GTK_WIDGET(verbEntry));
+
+    // currentPos is now after insertion. Put cursor there.
+    //
+    gtk_editable_set_position(GTK_EDITABLE(verbEntry), currentPos);
+}
+
+
+static
+void
+addAccentedCharButton(GtkWidget *box, const char *utf8AccentedChar)
+{
+    GtkWidget *button = gtk_button_new_with_mnemonic(utf8AccentedChar);
+    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		G_CALLBACK(onAccentedCharClicked),
+		reinterpret_cast<gpointer>(const_cast<char *>(utf8AccentedChar)));
+}
+
+
 static
 void
 showResultWin()
@@ -547,7 +808,7 @@ showResultWin()
 	gtk_window_set_title(GTK_WINDOW(resultWin), PACKAGE_FULL_NAME);
 
 	// Get the size of the results window from the saved configuration, if available.
-	gint resultWinWidth, resultWinHeight;
+	gint resultWinWidth = 0, resultWinHeight = 0;
 	if (!getResultWindowSize(resultWinWidth, resultWinHeight))  // if failed
 	    resultWinWidth = 610, resultWinHeight = 530;  // use default values
 	gtk_window_set_default_size(GTK_WINDOW(resultWin), resultWinWidth, resultWinHeight);
@@ -590,17 +851,17 @@ showResultWin()
 	gtk_box_pack_start(GTK_BOX(promptBox), prompt, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(promptBox), verbEntry, TRUE, TRUE, 0);
 
-	conjButton = gtk_button_new_with_mnemonic(_("_Conjugate"));
+	conjButton = gtk_button_new_with_mnemonic(_("Conjuga_te"));
 	gtk_widget_set_sensitive(GTK_WIDGET(conjButton), false);
 	gtk_box_pack_start(GTK_BOX(promptBox), conjButton, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(conjButton), "clicked",
 				    G_CALLBACK(onConjugateButton), NULL);
 
-	saveButton = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
-	gtk_widget_set_sensitive(GTK_WIDGET(saveButton), false);
-	gtk_box_pack_start(GTK_BOX(promptBox), saveButton, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(saveButton), "clicked",
-				    G_CALLBACK(onSaveButton), NULL);
+	saveAsButton = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
+	gtk_widget_set_sensitive(GTK_WIDGET(saveAsButton), false);
+	gtk_box_pack_start(GTK_BOX(promptBox), saveAsButton, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(saveAsButton), "clicked",
+				    G_CALLBACK(onSaveAsButton), NULL);
 
 
 	/*
@@ -618,6 +879,26 @@ showResultWin()
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showPronounsCB),
 					strcmp(showPronouns.get(), "1") == 0);
 
+	gtk_box_pack_start(GTK_BOX(optionsBox), gtk_vseparator_new(), FALSE, FALSE, 0);
+
+	// Provide buttons that add accented characters to the Verb text field.
+	// The \x codes are in UTF-8.
+	//
+	//GtkWidget *nameLabel = newLabel(_("Accented characters:"), FALSE, 0.5);
+	//gtk_box_pack_start(GTK_BOX(optionsBox), nameLabel, FALSE, FALSE, 0);
+	addAccentedCharButton(optionsBox, "\xc3\xa0"); // in HTML: &agrave;
+	addAccentedCharButton(optionsBox, "\xc3\xa2"); // in HTML: &acirc;
+	addAccentedCharButton(optionsBox, "\xc3\xa7"); // in HTML: &ccedil;
+	addAccentedCharButton(optionsBox, "\xc3\xa9"); // in HTML: &eacute;
+	addAccentedCharButton(optionsBox, "\xc3\xa8"); // in HTML: &egrave;
+	addAccentedCharButton(optionsBox, "\xc3\xaa"); // in HTML: &ecirc;
+	addAccentedCharButton(optionsBox, "\xc3\xab"); // in HTML: &euml;
+	addAccentedCharButton(optionsBox, "\xc3\xae"); // in HTML: &icirc;
+	addAccentedCharButton(optionsBox, "\xc3\xaf"); // in HTML: &iuml;
+	addAccentedCharButton(optionsBox, "\xc3\xb4"); // in HTML: &ocirc;
+	addAccentedCharButton(optionsBox, "\xc3\xb9"); // in HTML: &ugrave;
+	addAccentedCharButton(optionsBox, "\xc3\xbb"); // in HTML: &ucirc;
+
 	/*
 	    Create a notebook that receives the conjugations.
 	*/
@@ -682,7 +963,7 @@ clearResultNotebook()
 	}
     }
 
-    gtk_widget_set_sensitive(GTK_WIDGET(saveButton), false);
+    gtk_widget_set_sensitive(GTK_WIDGET(saveAsButton), false);
 }
 
 
@@ -822,7 +1103,7 @@ processText(const string &utf8UserText)
     }
 
 
-    gtk_widget_set_sensitive(GTK_WIDGET(saveButton), numPages > 0);
+    gtk_widget_set_sensitive(GTK_WIDGET(saveAsButton), numPages > 0);
 
 
     if (resultNotebook != NULL)
