@@ -1,4 +1,4 @@
-/*  $Id: conjugation.h,v 1.3 2006/07/08 19:20:05 sarrazip Exp $
+/*  $Id: conjugation.h,v 1.4 2009/10/31 02:02:05 sarrazip Exp $
     conjugation.h - Generic conjugation interface
 
     verbiste - French conjugation system
@@ -72,31 +72,6 @@ void getConjugation(const verbiste::FrenchVerbDictionary &fvd,
 std::string getTenseNameForTableCell(int row, int col);
 
 
-/** Composes the text of the conjugation in a certain tense.
-    The words of this conjugation that match a certain user text
-    are marked specially.
-
-    @param	tense			structure that represents a certain
-					tense of a certain verb
-    @param	lowerCaseLatin1UserText	conjugated verb as entered by the
-					user, but converted to lower-case,
-					and assumed to be in Latin-1
-    @param	openMark		ASCII string to be used before verbs
-					that match lowerCaseLatin1UserText
-					(e.g., "<b>")
-    @param	closeMark		ASCII string to be used after verbs
-					that match lowerCaseLatin1UserText
-					(e.g., "</b>")
-    @param	inflectionList		reference to an initially empty list
-					which will receive Latin-1 strings
-*/
-void createTableCellText(const VVS &tense,
-			    const std::string &lowerCaseULatin1serText,
-			    const std::string &openMark,
-			    const std::string &closeMark,
-			    VS &inflectionList);
-
-
 /** Composes the text of the conjugation in a certain tense.
     The words of this conjugation that match a certain user text
     are marked specially.
