@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.cpp,v 1.22 2009/03/10 02:59:36 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.cpp,v 1.24 2009/10/31 02:02:05 sarrazip Exp $
     FrenchVerbDictionary.cpp - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -127,7 +127,58 @@ convertTenseName(const xmlChar *tenseName)
 }
 
 
-FrenchVerbDictionary::FrenchVerbDictionary() throw (logic_error)
+// Latin-1 to ASCII conversion table (codes 0xC0 to 0xFF).
+// Some characters have bogus translations, but they are not used in French.
+//
+static const char *accentRemovalTable =
+	    "AAAAAA_CEEEEIIII"
+	    "DNOOOOOxOUUUUYbB"
+	    "aaaaaa-ceeeeiiii"
+	    "dnooooo/ouuuuyby";
+
+
+static char
+removeAccent(char c)
+{
+    unsigned char u = static_cast<unsigned char>(c);
+    if (u >= 0xC0)
+	u = accentRemovalTable[u - 0xC0];
+    return static_cast<char>(u);
+}
+
+
+void
+FrenchVerbDictionary::formUnaccentedVariants(const string &latin1String,
+						size_t index,
+						vector<string> &variants)
+{
+    for ( ; index < latin1String.length(); ++index)
+    {
+	char ch = latin1String[index];
+	char unacc = removeAccent(ch);
+	if (ch != unacc)
+	{
+	    string copy = latin1String;
+	    copy[index] = unacc;
+	    variants.push_back(copy);
+	    formUnaccentedVariants(copy, index + 1, variants);
+	}
+    }
+}
+
+
+string
+FrenchVerbDictionary::removeAccents(const string &latin1String)
+{
+    string result;
+    result.reserve(latin1String.length());
+    for (size_t i = 0; i < latin1String.length(); ++i)
+	result += removeAccent(latin1String[i]);
+    return result;
+}
+
+
+FrenchVerbDictionary::FrenchVerbDictionary(bool includeWithoutAccents) throw (logic_error)
   : conjugSys(),
     knownVerbs(),
     inflectionTable(),
@@ -143,26 +194,28 @@ FrenchVerbDictionary::FrenchVerbDictionary() throw (logic_error)
     string conjFN  = libdatadir + string("/") + "conjugation-fr.xml";
     string verbsFN = libdatadir + string("/") + "verbs-fr.xml";
 
-    init(conjFN, verbsFN);
+    init(conjFN, verbsFN, includeWithoutAccents);
 }
 
 
 FrenchVerbDictionary::FrenchVerbDictionary(
 				const string &conjugationFilename,
-				const string &verbsFilename)
+				const string &verbsFilename,
+				bool includeWithoutAccents)
 					throw (logic_error)
   : conjugSys(),
     knownVerbs(),
     inflectionTable(),
     verbTrie(*this)
 {
-    init(conjugationFilename, verbsFilename);
+    init(conjugationFilename, verbsFilename, includeWithoutAccents);
 }
 
 
 void
 FrenchVerbDictionary::init(const string &conjugationFilename,
-			    const string &verbsFilename)
+			    const string &verbsFilename,
+			    bool includeWithoutAccents)
 					throw (logic_error)
 {
     toUTF8 = iconv_open("UTF-8", "ISO-8859-1");
@@ -181,14 +234,15 @@ FrenchVerbDictionary::init(const string &conjugationFilename,
 	    latin1TolowerTable[i] = char(i);
     }
 
-    loadConjugationDatabase(conjugationFilename.c_str());
-    loadVerbDatabase(verbsFilename.c_str());
+    loadConjugationDatabase(conjugationFilename.c_str(), includeWithoutAccents);
+    loadVerbDatabase(verbsFilename.c_str(), includeWithoutAccents);
 }
 
 
 void
 FrenchVerbDictionary::loadConjugationDatabase(
-				const char *conjugationFilename)
+				const char *conjugationFilename,
+				bool includeWithoutAccents)
 					throw (logic_error)
 {
     if (conjugationFilename == NULL)
@@ -198,13 +252,14 @@ FrenchVerbDictionary::loadConjugationDatabase(
     if (!conjDoc)
 	throw logic_error("could not parse " + string(conjugationFilename));
 
-    readConjugation(conjDoc.get());
+    readConjugation(conjDoc.get(), includeWithoutAccents);
 }
 
 
 void
 FrenchVerbDictionary::loadVerbDatabase(
-				const char *verbsFilename)
+				const char *verbsFilename,
+				bool includeWithoutAccents)
 					throw (logic_error)
 {
     if (verbsFilename == NULL)
@@ -214,12 +269,12 @@ FrenchVerbDictionary::loadVerbDatabase(
     if (!verbsDoc)
 	throw logic_error("could not parse " + string(verbsFilename));
 
-    readVerbs(verbsDoc.get());
+    readVerbs(verbsDoc.get(), includeWithoutAccents);
 }
 
 
 void
-FrenchVerbDictionary::readConjugation(xmlDocPtr doc) throw(logic_error)
+FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents) throw(logic_error)
 {
     xmlNodePtr rootNodePtr = xmlDocGetRootElement(doc);
 
@@ -291,22 +346,37 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc) throw(logic_error)
 		    theTenseSpec.push_back(PersonSpec());
 		    PersonSpec &thePersonSpec = theTenseSpec.back();
 
-		    // For each inflection for this person:
+		    // For each variant for this person:
 		    // (Note that most persons of most verbs have only
-		    // on inflection.)
+		    // one variant.)
 		    for (xmlNodePtr inf = person->xmlChildrenNode;
 					inf != NULL;
 					inf = inf->next)
 		    {
 			string variant = getLatin1XmlNodeText(
 						    doc, inf->xmlChildrenNode);
-			thePersonSpec.push_back(variant);
+			thePersonSpec.push_back(InflectionSpec(variant, true));
 
 			ModeTensePersonNumber mtpn(
 				(char *) mode->name,
 				(char *) tense->name,
-				personCounter);
+				personCounter,
+				true);
 			ti[variant].push_back(mtpn);
+
+			if (includeWithoutAccents)
+			{
+			    // Also include versions where some of all accents are missing.
+			    vector<string> unaccentedVariants;
+			    formUnaccentedVariants(variant, 0, unaccentedVariants);
+			    for (vector<string>::const_iterator it = unaccentedVariants.begin();
+								it != unaccentedVariants.end(); ++it)
+			    {
+				thePersonSpec.push_back(InflectionSpec(*it, false));
+				mtpn.correct = false;  // 'false' marks this spelling as incorrect.
+				ti[*it].push_back(mtpn);
+			    }
+			}
 		    }
 		}
 	    }
@@ -338,7 +408,9 @@ FrenchVerbDictionary::getLatin1XmlProp(xmlNodePtr node, const char *propName)
 
 
 void
-FrenchVerbDictionary::readVerbs(xmlDocPtr doc) throw(logic_error)
+FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
+				bool includeWithoutAccents)
+						throw(logic_error)
 {
     xmlNodePtr rootNodePtr = xmlDocGetRootElement(doc);
 
@@ -381,6 +453,18 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc) throw(logic_error)
 
 	knownVerbs[infinitive] = tname;
 
+	if (includeWithoutAccents)
+	{
+	    // Also include versions where some of all accents are missing.
+	    vector<string> unaccentedVariants;
+	    formUnaccentedVariants(infinitive, 0, unaccentedVariants);
+	    for (vector<string>::const_iterator it = unaccentedVariants.begin();
+						it != unaccentedVariants.end(); ++it)
+	    {
+		knownVerbs[*it] = tname;
+	    }
+	}
+
 	// <aspirate-h>: If this verb starts with an aspirate h, remember it:
 	if (t->next != NULL && t->next->next != NULL)
 	    aspirateHVerbs.insert(infinitive);
@@ -394,19 +478,44 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc) throw(logic_error)
 
 	string verbRadical(infinitive, 0, lenInfinitive - lenTermination);
 
-	vector<string> **templateListPtr =
-				verbTrie.getUserDataPointer(verbRadical);
-	assert(templateListPtr != NULL);
-	if (*templateListPtr == NULL)
+	insertVerbRadicalInTrie(verbRadical, tname);
+
+	if (includeWithoutAccents)
 	{
-	    //cerr << "new verbRadical: '" << verbRadical << "'\n";
-	    *templateListPtr = new vector<string>();
+	    // Also include versions where some of all accents are missing.
+	    vector<string> unaccentedVariants;
+	    formUnaccentedVariants(verbRadical, 0, unaccentedVariants);
+	    for (vector<string>::const_iterator it = unaccentedVariants.begin();
+						it != unaccentedVariants.end(); ++it)
+	    {
+		insertVerbRadicalInTrie(*it, tname);
+	    }
 	}
-	(*templateListPtr)->push_back(tname);
     }
 }
 
 
+void
+FrenchVerbDictionary::insertVerbRadicalInTrie(
+			const string &verbRadical, const string &tname)
+{
+    vector<string> **templateListPtr =
+			    verbTrie.getUserDataPointer(verbRadical);
+    assert(templateListPtr != NULL);
+
+    // If a new entry was created for 'verbRadical', then the associated
+    // user data pointer is null.  Make this pointer point to a new,
+    // empty vector of template names.
+    //
+    if (*templateListPtr == NULL)
+	*templateListPtr = new vector<string>();
+
+    // Associate the given template name to the given verb radical.
+    //
+    (*templateListPtr)->push_back(tname);
+}
+
+
 FrenchVerbDictionary::~FrenchVerbDictionary()
 {
     iconv_close(toLatin1);
@@ -767,8 +876,16 @@ FrenchVerbDictionary::generateTense(const string &radical,
 	dest.push_back(vector<string>());
 	for (PersonSpec::const_iterator i = p->begin(); i != p->end(); i++)
 	{
-	    string pronoun;
-	    string v = radical + *i;
+	    // Do not return spellings that are marked incorrect.
+	    // They are in the knowledge base only to allow
+	    // error-tolerant searches.
+	    //
+	    if (!(*i).isCorrect)
+		continue;
+
+	    string pronoun;  // no pronoun by default
+
+	    string v = radical + (*i).inflection;
 
 	    if (includePronouns)
 	    {
