@@ -1,4 +1,4 @@
-/*  $Id: Trie.cpp,v 1.6 2005/03/13 04:03:26 sarrazip Exp $
+/*  $Id: Trie.cpp,v 1.7 2009/10/31 02:02:05 sarrazip Exp $
     Trie.cpp - Tree structure for string storage
 
     verbiste - French conjugation system
@@ -146,7 +146,7 @@ Trie<T>::get(const string &key) const
     if (key.empty())
 	return lambda;
 
-    Descriptor *d = ((Trie<T> *) this)->getDesc(firstRow, key, 0, false, true);
+    Descriptor *d = const_cast<Trie<T> *>(this)->getDesc(firstRow, key, 0, false, true);
     return (d != NULL ? d->userData : NULL);
 }
 
@@ -177,6 +177,9 @@ Trie<T>::getUserDataPointer(const string &key)
     if (key.empty())
 	return &lambda;
 
+    // Get descriptor associated with 'key' (and create a new entry
+    // if the key is not known).
+    //
     Descriptor *d = getDesc(firstRow, key, 0, true, false);
     assert(d != NULL);
     return &d->userData;
@@ -211,7 +214,7 @@ Trie<T>::getDesc(Row *row,
 			key, index + 1, create, callFoundPrefixCallback);
     }
 
-    if (index + 1 == key.length())
+    if (index + 1 == key.length())  // if reached end of key
     {
 	if (callFoundPrefixCallback && pd->userData != NULL)
 	    onFoundPrefixWithUserData(key, index + 1, pd->userData);
